@@ -2,7 +2,7 @@
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2005 Wim Taymans <wim@fluendo.com>
  *
- * gstbaseaudiosink.c: 
+ * gstbaseaudiosink.c:
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -56,6 +56,12 @@ struct _GstBaseAudioSinkPrivate
   gboolean sync_latency;
 
   GstClockTime eos_time;
+  gint eos_rendering;
+
+  gboolean do_time_offset;
+  /* number of microseconds we alow timestamps or clock slaving to drift
+   * before resyncing */
+  guint64 drift_tolerance;
 };
 
 /* BaseAudioSink signals and args */
@@ -67,7 +73,7 @@ enum
 
 /* we tollerate half a second diff before we start resyncing. This
  * should be enough to compensate for various rounding errors in the timestamp
- * and sample offset position. 
+ * and sample offset position.
  * This is an emergency resync fallback since buffers marked as DISCONT will
  * always lock to the correct timestamp immediatly and buffers not marked as
  * DISCONT are contiguous by definition.
@@ -83,32 +89,43 @@ enum
 /* FIXME, enable pull mode when clock slaving and trick modes are figured out */
 #define DEFAULT_CAN_ACTIVATE_PULL FALSE
 
+/* when timestamps or clock slaving drift for more than 20ms we resync. This is
+ * a reasonable default */
+#define DEFAULT_DRIFT_TOLERANCE   ((40 * GST_MSECOND) / GST_USECOND)
+
 enum
 {
   PROP_0,
+
   PROP_BUFFER_TIME,
   PROP_LATENCY_TIME,
   PROP_PROVIDE_CLOCK,
   PROP_SLAVE_METHOD,
-  PROP_CAN_ACTIVATE_PULL
+  PROP_CAN_ACTIVATE_PULL,
+  PROP_DRIFT_TOLERANCE,
+
+  PROP_LAST
 };
 
 GType
 gst_base_audio_sink_slave_method_get_type (void)
 {
-  static GType slave_method_type = 0;
+  static volatile gsize slave_method_type = 0;
   static const GEnumValue slave_method[] = {
-    {GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE, "Resampling slaving", "resample"},
-    {GST_BASE_AUDIO_SINK_SLAVE_SKEW, "Skew slaving", "skew"},
-    {GST_BASE_AUDIO_SINK_SLAVE_NONE, "No slaving", "none"},
+    {GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE, "GST_BASE_AUDIO_SINK_SLAVE_RESAMPLE",
+        "resample"},
+    {GST_BASE_AUDIO_SINK_SLAVE_SKEW, "GST_BASE_AUDIO_SINK_SLAVE_SKEW", "skew"},
+    {GST_BASE_AUDIO_SINK_SLAVE_NONE, "GST_BASE_AUDIO_SINK_SLAVE_NONE", "none"},
     {0, NULL, NULL},
   };
 
-  if (!slave_method_type) {
-    slave_method_type =
+  if (g_once_init_enter (&slave_method_type)) {
+    GType tmp =
         g_enum_register_static ("GstBaseAudioSinkSlaveMethod", slave_method);
+    g_once_init_leave (&slave_method_type, tmp);
   }
-  return slave_method_type;
+
+  return (GType) slave_method_type;
 }
 
 
@@ -175,11 +192,9 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
 
   g_type_class_add_private (klass, sizeof (GstBaseAudioSinkPrivate));
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_base_audio_sink_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_base_audio_sink_get_property);
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_base_audio_sink_dispose);
+  gobject_class->set_property = gst_base_audio_sink_set_property;
+  gobject_class->get_property = gst_base_audio_sink_get_property;
+  gobject_class->dispose = gst_base_audio_sink_dispose;
 
   g_object_class_install_property (gobject_class, PROP_BUFFER_TIME,
       g_param_spec_int64 ("buffer-time", "Buffer Time",
@@ -208,6 +223,19 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
       g_param_spec_boolean ("can-activate-pull", "Allow Pull Scheduling",
           "Allow pull-based scheduling", DEFAULT_CAN_ACTIVATE_PULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  /**
+   * GstBaseAudioSink:drift-tolerance
+   *
+   * Controls the amount of time in milliseconds that timestamps or clocks are allowed
+   * to drift before resynchronisation happens.
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_DRIFT_TOLERANCE,
+      g_param_spec_int64 ("drift-tolerance", "Drift Tolerance",
+          "Tolerance for timestamp and clock drift in microseconds", 1,
+          G_MAXINT64, DEFAULT_DRIFT_TOLERANCE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_change_state);
@@ -231,12 +259,15 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
    * thread-safety in GObject */
   g_type_class_ref (GST_TYPE_AUDIO_CLOCK);
   g_type_class_ref (GST_TYPE_RING_BUFFER);
+
 }
 
 static void
 gst_base_audio_sink_init (GstBaseAudioSink * baseaudiosink,
     GstBaseAudioSinkClass * g_class)
 {
+  GstPluginFeature *feature;
+
   baseaudiosink->priv = GST_BASE_AUDIO_SINK_GET_PRIVATE (baseaudiosink);
 
   baseaudiosink->buffer_time = DEFAULT_BUFFER_TIME;
@@ -249,10 +280,30 @@ gst_base_audio_sink_init (GstBaseAudioSink * baseaudiosink,
 
   GST_BASE_SINK (baseaudiosink)->can_activate_push = TRUE;
   GST_BASE_SINK (baseaudiosink)->can_activate_pull = DEFAULT_CAN_ACTIVATE_PULL;
+  baseaudiosink->priv->drift_tolerance = DEFAULT_DRIFT_TOLERANCE;
 
   /* install some custom pad_query functions */
   gst_pad_set_query_function (GST_BASE_SINK_PAD (baseaudiosink),
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_query_pad));
+
+  baseaudiosink->priv->do_time_offset = TRUE;
+
+  /* check the factory, pulsesink < 0.10.17 does the timestamp offset itself so
+   * we should not do ourselves */
+  feature =
+      GST_PLUGIN_FEATURE_CAST (GST_ELEMENT_CLASS (g_class)->elementfactory);
+  GST_DEBUG ("created from factory %p", feature);
+
+  /* HACK for old pulsesink that did the time_offset themselves */
+  if (feature) {
+    if (strcmp (gst_plugin_feature_get_name (feature), "pulsesink") == 0) {
+      if (!gst_plugin_feature_check_version (feature, 0, 10, 17)) {
+        /* we're dealing with an old pulsesink, we need to disable time corection */
+        GST_DEBUG ("disable time offset");
+        baseaudiosink->priv->do_time_offset = FALSE;
+      }
+    }
+  }
 }
 
 static void
@@ -262,9 +313,11 @@ gst_base_audio_sink_dispose (GObject * object)
 
   sink = GST_BASE_AUDIO_SINK (object);
 
-  if (sink->provided_clock)
+  if (sink->provided_clock) {
+    gst_audio_clock_invalidate (sink->provided_clock);
     gst_object_unref (sink->provided_clock);
-  sink->provided_clock = NULL;
+    sink->provided_clock = NULL;
+  }
 
   if (sink->ringbuffer) {
     gst_object_unparent (GST_OBJECT_CAST (sink->ringbuffer));
@@ -474,7 +527,7 @@ gst_base_audio_sink_get_time (GstClock * clock, GstBaseAudioSink * sink)
  * @sink: a #GstBaseAudioSink
  * @provide: new state
  *
- * Controls whether @sink will provide a clock or not. If @provide is %TRUE, 
+ * Controls whether @sink will provide a clock or not. If @provide is %TRUE,
  * gst_element_provide_clock() will return a clock that reflects the datarate
  * of @sink. If @provide is %FALSE, gst_element_provide_clock() will return NULL.
  *
@@ -521,7 +574,7 @@ gst_base_audio_sink_get_provide_clock (GstBaseAudioSink * sink)
  * @sink: a #GstBaseAudioSink
  * @method: the new slave method
  *
- * Controls how clock slaving will be performed in @sink. 
+ * Controls how clock slaving will be performed in @sink.
  *
  * Since: 0.10.16
  */
@@ -560,6 +613,51 @@ gst_base_audio_sink_get_slave_method (GstBaseAudioSink * sink)
   return result;
 }
 
+
+/**
+ * gst_base_audio_sink_set_drift_tolerance:
+ * @sink: a #GstBaseAudioSink
+ * @drift_tolerance: the new drift tolerance in microseconds
+ *
+ * Controls the sink's drift tolerance.
+ *
+ * Since: 0.10.31
+ */
+void
+gst_base_audio_sink_set_drift_tolerance (GstBaseAudioSink * sink,
+    gint64 drift_tolerance)
+{
+  g_return_if_fail (GST_IS_BASE_AUDIO_SINK (sink));
+
+  GST_OBJECT_LOCK (sink);
+  sink->priv->drift_tolerance = drift_tolerance;
+  GST_OBJECT_UNLOCK (sink);
+}
+
+/**
+ * gst_base_audio_sink_get_drift_tolerance
+ * @sink: a #GstBaseAudioSink
+ *
+ * Get the current drift tolerance, in microseconds, used by @sink.
+ *
+ * Returns: The current drift tolerance used by @sink.
+ *
+ * Since: 0.10.31
+ */
+gint64
+gst_base_audio_sink_get_drift_tolerance (GstBaseAudioSink * sink)
+{
+  gint64 result;
+
+  g_return_val_if_fail (GST_IS_BASE_AUDIO_SINK (sink), -1);
+
+  GST_OBJECT_LOCK (sink);
+  result = sink->priv->drift_tolerance;
+  GST_OBJECT_UNLOCK (sink);
+
+  return result;
+}
+
 static void
 gst_base_audio_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -584,6 +682,9 @@ gst_base_audio_sink_set_property (GObject * object, guint prop_id,
     case PROP_CAN_ACTIVATE_PULL:
       GST_BASE_SINK (sink)->can_activate_pull = g_value_get_boolean (value);
       break;
+    case PROP_DRIFT_TOLERANCE:
+      gst_base_audio_sink_set_drift_tolerance (sink, g_value_get_int64 (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -614,6 +715,9 @@ gst_base_audio_sink_get_property (GObject * object, guint prop_id,
     case PROP_CAN_ACTIVATE_PULL:
       g_value_set_boolean (value, GST_BASE_SINK (sink)->can_activate_pull);
       break;
+    case PROP_DRIFT_TOLERANCE:
+      g_value_set_int64 (value, gst_base_audio_sink_get_drift_tolerance (sink));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -625,6 +729,7 @@ gst_base_audio_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
 {
   GstBaseAudioSink *sink = GST_BASE_AUDIO_SINK (bsink);
   GstRingBufferSpec *spec;
+  GstClockTime now;
 
   if (!sink->ringbuffer)
     return FALSE;
@@ -633,6 +738,11 @@ gst_base_audio_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
 
   GST_DEBUG_OBJECT (sink, "release old ringbuffer");
 
+  /* get current time, updates the last_time */
+  now = gst_clock_get_time (sink->provided_clock);
+
+  GST_DEBUG_OBJECT (sink, "time was %" GST_TIME_FORMAT, GST_TIME_ARGS (now));
+
   /* release old ringbuffer */
   gst_ring_buffer_pause (sink->ringbuffer);
   gst_ring_buffer_activate (sink->ringbuffer, FALSE);
@@ -658,7 +768,7 @@ gst_base_audio_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     gst_ring_buffer_activate (sink->ringbuffer, TRUE);
   }
 
-  /* calculate actual latency and buffer times. 
+  /* calculate actual latency and buffer times.
    * FIXME: In 0.11, store the latency_time internally in ns */
   spec->latency_time = gst_util_uint64_scale (spec->segsize,
       (GST_SECOND / GST_USECOND), spec->rate * spec->bytes_per_sample);
@@ -729,6 +839,10 @@ gst_base_audio_sink_drain (GstBaseAudioSink * sink)
   if (!sink->ringbuffer->spec.rate)
     return TRUE;
 
+  /* if PLAYING is interrupted,
+   * arrange to have clock running when going to PLAYING again */
+  g_atomic_int_set (&sink->priv->eos_rendering, 1);
+
   /* need to start playback before we can drain, but only when
    * we have successfully negotiated a format and thus acquired the
    * ringbuffer. */
@@ -746,6 +860,7 @@ gst_base_audio_sink_drain (GstBaseAudioSink * sink)
 
     GST_DEBUG_OBJECT (sink, "drained audio");
   }
+  g_atomic_int_set (&sink->priv->eos_rendering, 0);
   return TRUE;
 }
 
@@ -925,8 +1040,8 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
 {
   GstClockTime cinternal, cexternal, crate_num, crate_denom;
   GstClockTime etime, itime;
-  GstClockTimeDiff skew, segtime, segtime2;
-  gint segsamples;
+  GstClockTimeDiff skew, mdrift, mdrift2;
+  gint driftsamples;
   gint64 last_align;
 
   /* get calibration parameters to compensate for offsets */
@@ -936,6 +1051,7 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
   /* sample clocks and figure out clock skew */
   etime = gst_clock_get_time (GST_ELEMENT_CLOCK (sink));
   itime = gst_audio_clock_get_time (sink->provided_clock);
+  itime = gst_audio_clock_adjust (sink->provided_clock, itime);
 
   GST_DEBUG_OBJECT (sink,
       "internal %" GST_TIME_FORMAT " external %" GST_TIME_FORMAT
@@ -963,56 +1079,52 @@ gst_base_audio_sink_skew_slaving (GstBaseAudioSink * sink,
       GST_TIME_FORMAT " skew %" G_GINT64_FORMAT " avg %" G_GINT64_FORMAT,
       GST_TIME_ARGS (itime), GST_TIME_ARGS (etime), skew, sink->priv->avg_skew);
 
-  /* the max drift we allow is the length of a segment */
-  segtime = sink->ringbuffer->spec.latency_time * 1000;
-  segtime2 = segtime / 2;
+  /* the max drift we allow */
+  mdrift = sink->priv->drift_tolerance * 1000;
+  mdrift2 = mdrift / 2;
 
   /* adjust playout pointer based on skew */
-  if (sink->priv->avg_skew > segtime2) {
+  if (sink->priv->avg_skew > mdrift2) {
     /* master is running slower, move internal time forward */
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " > %" G_GINT64_FORMAT,
-        sink->priv->avg_skew, segtime2);
-    cexternal = cexternal > segtime ? cexternal - segtime : 0;
-    sink->priv->avg_skew -= segtime;
+        sink->priv->avg_skew, mdrift2);
+    cexternal = cexternal > mdrift ? cexternal - mdrift : 0;
+    sink->priv->avg_skew -= mdrift;
 
-    segsamples =
-        sink->ringbuffer->spec.segsize /
-        sink->ringbuffer->spec.bytes_per_sample;
+    driftsamples = (sink->ringbuffer->spec.rate * mdrift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
     /* if we were aligning in the wrong direction or we aligned more than what we
      * will correct, resync */
-    if (last_align < 0 || last_align > segsamples)
+    if (last_align < 0 || last_align > driftsamples)
       sink->next_sample = -1;
 
     GST_DEBUG_OBJECT (sink,
-        "last_align %" G_GINT64_FORMAT " segsamples %u, next %"
-        G_GUINT64_FORMAT, last_align, segsamples, sink->next_sample);
+        "last_align %" G_GINT64_FORMAT " driftsamples %u, next %"
+        G_GUINT64_FORMAT, last_align, driftsamples, sink->next_sample);
 
     gst_clock_set_calibration (sink->provided_clock, cinternal, cexternal,
         crate_num, crate_denom);
-  } else if (sink->priv->avg_skew < -segtime2) {
+  } else if (sink->priv->avg_skew < -mdrift2) {
     /* master is running faster, move external time forwards */
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " < %" G_GINT64_FORMAT,
-        sink->priv->avg_skew, -segtime2);
-    cexternal += segtime;
-    sink->priv->avg_skew += segtime;
+        sink->priv->avg_skew, -mdrift2);
+    cexternal += mdrift;
+    sink->priv->avg_skew += mdrift;
 
-    segsamples =
-        sink->ringbuffer->spec.segsize /
-        sink->ringbuffer->spec.bytes_per_sample;
+    driftsamples = (sink->ringbuffer->spec.rate * mdrift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
     /* if we were aligning in the wrong direction or we aligned more than what we
      * will correct, resync */
-    if (last_align > 0 || -last_align > segsamples)
+    if (last_align > 0 || -last_align > driftsamples)
       sink->next_sample = -1;
 
     GST_DEBUG_OBJECT (sink,
-        "last_align %" G_GINT64_FORMAT " segsamples %u, next %"
-        G_GUINT64_FORMAT, last_align, segsamples, sink->next_sample);
+        "last_align %" G_GINT64_FORMAT " driftsamples %u, next %"
+        G_GUINT64_FORMAT, last_align, driftsamples, sink->next_sample);
 
     gst_clock_set_calibration (sink->provided_clock, cinternal, cexternal,
         crate_num, crate_denom);
@@ -1081,7 +1193,7 @@ gst_base_audio_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
 {
   GstClock *clock;
   GstClockReturn status;
-  GstClockTime time;
+  GstClockTime time, render_delay;
   GstFlowReturn ret;
   GstBaseAudioSink *sink;
   GstClockTime itime, etime;
@@ -1111,11 +1223,20 @@ gst_base_audio_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
     time = sink->priv->us_latency;
     GST_OBJECT_UNLOCK (sink);
 
+    /* Renderdelay is added onto our own latency, and needs
+     * to be subtracted as well */
+    render_delay = gst_base_sink_get_render_delay (bsink);
+
+    if (G_LIKELY (time > render_delay))
+      time -= render_delay;
+    else
+      time = 0;
+
     /* preroll done, we can sync since we are in PLAYING now. */
     GST_DEBUG_OBJECT (sink, "possibly waiting for clock to reach %"
         GST_TIME_FORMAT, GST_TIME_ARGS (time));
 
-    /* wait for the clock, this can be interrupted because we got shut down or 
+    /* wait for the clock, this can be interrupted because we got shut down or
      * we PAUSED. */
     status = gst_base_sink_wait_clock (bsink, time, &jitter);
 
@@ -1142,6 +1263,7 @@ gst_base_audio_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
    * time of the external clock) */
   etime = GST_ELEMENT_CAST (sink)->base_time + time;
   itime = gst_audio_clock_get_time (sink->provided_clock);
+  itime = gst_audio_clock_adjust (sink->provided_clock, itime);
 
   if (status == GST_CLOCK_EARLY) {
     /* when we prerolled late, we have to take into account the lateness */
@@ -1220,6 +1342,8 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   gboolean sync, slaved, align_next;
   GstFlowReturn ret;
   GstSegment clip_seg;
+  gint64 time_offset;
+  gint64 maxdrift;
 
   sink = GST_BASE_AUDIO_SINK (bsink);
 
@@ -1310,8 +1434,8 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   }
 
   /* samples should be rendered based on their timestamp. All samples
-   * arriving before the segment.start or after segment.stop are to be 
-   * thrown away. All samples should also be clipped to the segment 
+   * arriving before the segment.start or after segment.stop are to be
+   * thrown away. All samples should also be clipped to the segment
    * boundaries */
   if (!gst_segment_clip (&clip_seg, GST_FORMAT_TIME, time, stop, &ctime,
           &cstop))
@@ -1401,6 +1525,26 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
         &render_start, &render_stop);
   }
 
+  GST_DEBUG_OBJECT (sink,
+      "final timestamps: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
+
+  /* bring to position in the ringbuffer */
+  if (sink->priv->do_time_offset) {
+    time_offset =
+        GST_AUDIO_CLOCK_CAST (sink->provided_clock)->abidata.ABI.time_offset;
+    GST_DEBUG_OBJECT (sink,
+        "time offset %" GST_TIME_FORMAT, GST_TIME_ARGS (time_offset));
+    if (render_start > time_offset)
+      render_start -= time_offset;
+    else
+      render_start = 0;
+    if (render_stop > time_offset)
+      render_stop -= time_offset;
+    else
+      render_stop = 0;
+  }
+
   /* and bring the time to the rate corrected offset in the buffer */
   render_start = gst_util_uint64_scale_int (render_start,
       ringbuf->spec.rate, GST_SECOND);
@@ -1440,26 +1584,20 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   else
     diff = sink->next_sample - sample_offset;
 
-  /* we tollerate half a second diff before we start resyncing. This
-   * should be enough to compensate for various rounding errors in the timestamp
-   * and sample offset position. We always resync if we got a discont anyway and
-   * non-discont should be aligned by definition. */
-  if (G_LIKELY (diff < ringbuf->spec.rate / DIFF_TOLERANCE)) {
+  /* calculate the max allowed drift in units of samples. By default this is
+   * 20ms and should be anough to compensate for timestamp rounding errors. */
+  maxdrift = (ringbuf->spec.rate * sink->priv->drift_tolerance) / GST_MSECOND;
+
+  if (G_LIKELY (diff < maxdrift)) {
     /* calc align with previous sample */
     align = sink->next_sample - sample_offset;
     GST_DEBUG_OBJECT (sink,
-        "align with prev sample, ABS (%" G_GINT64_FORMAT ") < %d", align,
-        ringbuf->spec.rate / DIFF_TOLERANCE);
+        "align with prev sample, ABS (%" G_GINT64_FORMAT ") < %"
+        G_GINT64_FORMAT, align, maxdrift);
   } else {
-    /* bring sample diff to seconds for error message */
-    diff = gst_util_uint64_scale_int (diff, GST_SECOND, ringbuf->spec.rate);
-    /* timestamps drifted apart from previous samples too much, we need to
-     * resync. We log this as an element warning. */
-    GST_ELEMENT_WARNING (sink, CORE, CLOCK,
-        ("Compensating for audio synchronisation problems"),
-        ("Unexpected discontinuity in audio timestamps of more "
-            "than half a second (%" GST_TIME_FORMAT "), resyncing",
-            GST_TIME_ARGS (diff)));
+    GST_DEBUG_OBJECT (sink,
+        "discont timestamp (%" G_GINT64_FORMAT ") >= %" G_GINT64_FORMAT, diff,
+        maxdrift);
     align = 0;
   }
   sink->priv->last_align = align;
@@ -1752,8 +1890,19 @@ gst_base_audio_sink_change_state (GstElement * element,
       sink->next_sample = -1;
       sink->priv->last_align = -1;
       sink->priv->eos_time = -1;
+      sink->priv->eos_rendering = 0;
       gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       gst_ring_buffer_may_start (sink->ringbuffer, FALSE);
+
+      /* Only post clock-provide messages if this is the clock that
+       * we've created. If the subclass has overriden it the subclass
+       * should post this messages whenever necessary */
+      if (sink->provided_clock && GST_IS_AUDIO_CLOCK (sink->provided_clock) &&
+          GST_AUDIO_CLOCK_CAST (sink->provided_clock)->func ==
+          (GstAudioClockGetTimeFunc) gst_base_audio_sink_get_time)
+        gst_element_post_message (element,
+            gst_message_new_clock_provide (GST_OBJECT_CAST (element),
+                sink->provided_clock, TRUE));
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_OBJECT_LOCK (sink);
@@ -1762,8 +1911,10 @@ gst_base_audio_sink_change_state (GstElement * element,
       GST_OBJECT_UNLOCK (sink);
 
       gst_ring_buffer_may_start (sink->ringbuffer, TRUE);
-      if (GST_BASE_SINK_CAST (sink)->pad_mode == GST_ACTIVATE_PULL) {
+      if (GST_BASE_SINK_CAST (sink)->pad_mode == GST_ACTIVATE_PULL ||
+          g_atomic_int_get (&sink->priv->eos_rendering)) {
         /* we always start the ringbuffer in pull mode immediatly */
+        /* sync rendering on eos needs running clock */
         gst_ring_buffer_start (sink->ringbuffer);
       }
       break;
@@ -1777,6 +1928,16 @@ gst_base_audio_sink_change_state (GstElement * element,
       GST_OBJECT_UNLOCK (sink);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      /* Only post clock-lost messages if this is the clock that
+       * we've created. If the subclass has overriden it the subclass
+       * should post this messages whenever necessary */
+      if (sink->provided_clock && GST_IS_AUDIO_CLOCK (sink->provided_clock) &&
+          GST_AUDIO_CLOCK_CAST (sink->provided_clock)->func ==
+          (GstAudioClockGetTimeFunc) gst_base_audio_sink_get_time)
+        gst_element_post_message (element,
+            gst_message_new_clock_lost (GST_OBJECT_CAST (element),
+                sink->provided_clock));
+
       /* make sure we unblock before calling the parent state change
        * so it can grab the STREAM_LOCK */
       gst_ring_buffer_set_flushing (sink->ringbuffer, TRUE);
