@@ -53,6 +53,18 @@ G_BEGIN_DECLS
  * @GST_VIDEO_FORMAT_v216: packed 4:2:2 16-bit YUV, Y0-U0-Y1-V1 order (Since: 0.10.24)
  * @GST_VIDEO_FORMAT_NV12: planar 4:2:0 YUV with interleaved UV plane (Since: 0.10.26)
  * @GST_VIDEO_FORMAT_NV21: planar 4:2:0 YUV with interleaved VU plane (Since: 0.10.26)
+ * @GST_VIDEO_FORMAT_GRAY8: 8-bit grayscale (Since: 0.10.29)
+ * @GST_VIDEO_FORMAT_GRAY16_BE: 16-bit grayscale, most significant byte first (Since: 0.10.29)
+ * @GST_VIDEO_FORMAT_GRAY16_LE: 16-bit grayscale, least significant byte first (Since: 0.10.29)
+ * @GST_VIDEO_FORMAT_v308: packed 4:4:4 YUV (Since: 0.10.29)
+ * @GST_VIDEO_FORMAT_Y800: same as GST_VIDEO_FORMAT_GRAY8 (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_Y16: same as GST_VIDEO_FORMAT_GRAY16_LE (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_RGB16: rgb 5-6-5 bits per component (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_BGR16: reverse rgb 5-6-5 bits per component (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_RGB15: rgb 5-5-5 bits per component (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_BGR15: reverse rgb 5-5-5 bits per component (Since: 0.10.30)
+ * @GST_VIDEO_FORMAT_UYVP: packed 10-bit 4:2:2 YUV (U0-Y0-V0-Y1 U2-Y2-V2-Y3 U4 ...) (Since: 0.10.31)
+ * @GST_VIDEO_FORMAT_A420: planar 4:4:2:0 AYUV (Since: 0.10.31)
  *
  * Enum value describing the most common video formats.
  */
@@ -80,7 +92,19 @@ typedef enum {
   GST_VIDEO_FORMAT_v210,
   GST_VIDEO_FORMAT_v216,
   GST_VIDEO_FORMAT_NV12,
-  GST_VIDEO_FORMAT_NV21
+  GST_VIDEO_FORMAT_NV21,
+  GST_VIDEO_FORMAT_GRAY8,
+  GST_VIDEO_FORMAT_GRAY16_BE,
+  GST_VIDEO_FORMAT_GRAY16_LE,
+  GST_VIDEO_FORMAT_v308,
+  GST_VIDEO_FORMAT_Y800,
+  GST_VIDEO_FORMAT_Y16,
+  GST_VIDEO_FORMAT_RGB16,
+  GST_VIDEO_FORMAT_BGR16,
+  GST_VIDEO_FORMAT_RGB15,
+  GST_VIDEO_FORMAT_BGR15,
+  GST_VIDEO_FORMAT_UYVP,
+  GST_VIDEO_FORMAT_A420
 } GstVideoFormat;
 
 #define GST_VIDEO_BYTE1_MASK_32  "0xFF000000"
@@ -101,21 +125,39 @@ typedef enum {
 #define GST_VIDEO_BYTE2_MASK_24_INT  0x0000FF00
 #define GST_VIDEO_BYTE3_MASK_24_INT  0x000000FF
 
-#define GST_VIDEO_RED_MASK_16 "0xf800"
-#define GST_VIDEO_GREEN_MASK_16 "0x07e0"
-#define GST_VIDEO_BLUE_MASK_16 "0x001f"
+#define GST_VIDEO_COMP1_MASK_16 "0xf800"
+#define GST_VIDEO_COMP2_MASK_16 "0x07e0"
+#define GST_VIDEO_COMP3_MASK_16 "0x001f"
 
-#define GST_VIDEO_RED_MASK_15 "0x7c00"
-#define GST_VIDEO_GREEN_MASK_15 "0x03e0"
-#define GST_VIDEO_BLUE_MASK_15 "0x001f"
+#define GST_VIDEO_COMP1_MASK_15 "0x7c00"
+#define GST_VIDEO_COMP2_MASK_15 "0x03e0"
+#define GST_VIDEO_COMP3_MASK_15 "0x001f"
 
-#define GST_VIDEO_RED_MASK_16_INT 0xf800
-#define GST_VIDEO_GREEN_MASK_16_INT 0x07e0
-#define GST_VIDEO_BLUE_MASK_16_INT 0x001f
+#define GST_VIDEO_COMP1_MASK_16_INT 0xf800
+#define GST_VIDEO_COMP2_MASK_16_INT 0x07e0
+#define GST_VIDEO_COMP3_MASK_16_INT 0x001f
 
-#define GST_VIDEO_RED_MASK_15_INT 0x7c00
-#define GST_VIDEO_GREEN_MASK_15_INT 0x03e0
-#define GST_VIDEO_BLUE_MASK_15_INT 0x001f
+#define GST_VIDEO_COMP1_MASK_15_INT 0x7c00
+#define GST_VIDEO_COMP2_MASK_15_INT 0x03e0
+#define GST_VIDEO_COMP3_MASK_15_INT 0x001f
+
+#ifndef GST_DISABLE_DEPRECATED
+#define GST_VIDEO_RED_MASK_16 GST_VIDEO_COMP1_MASK_16
+#define GST_VIDEO_GREEN_MASK_16 GST_VIDEO_COMP2_MASK_16
+#define GST_VIDEO_BLUE_MASK_16 GST_VIDEO_COMP3_MASK_16
+
+#define GST_VIDEO_RED_MASK_15 GST_VIDEO_COMP1_MASK_15
+#define GST_VIDEO_GREEN_MASK_15 GST_VIDEO_COMP2_MASK_15
+#define GST_VIDEO_BLUE_MASK_15 GST_VIDEO_COMP3_MASK_15
+
+#define GST_VIDEO_RED_MASK_16_INT GST_VIDEO_COMP1_MASK_16_INT
+#define GST_VIDEO_GREEN_MASK_16_INT GST_VIDEO_COMP2_MASK_16_INT
+#define GST_VIDEO_BLUE_MASK_16_INT GST_VIDEO_COMP3_MASK_16_INT
+
+#define GST_VIDEO_RED_MASK_15_INT GST_VIDEO_COMP1_MASK_15_INT
+#define GST_VIDEO_GREEN_MASK_15_INT GST_VIDEO_COMP2_MASK_15_INT
+#define GST_VIDEO_BLUE_MASK_15_INT GST_VIDEO_COMP3_MASK_15_INT
+#endif
 
 #define GST_VIDEO_SIZE_RANGE "(int) [ 1, max ]"
 #define GST_VIDEO_FPS_RANGE "(fraction) [ 0, max ]"
@@ -158,6 +200,30 @@ typedef enum {
     "height = " GST_VIDEO_SIZE_RANGE ", "                               \
     "framerate = " GST_VIDEO_FPS_RANGE
 
+#define __GST_VIDEO_CAPS_MAKE_16(R, G, B)                               \
+    "video/x-raw-rgb, "                                                 \
+    "bpp = (int) 16, "                                                  \
+    "depth = (int) 16, "                                                \
+    "endianness = (int) BYTE_ORDER, "                                   \
+    "red_mask = (int) " GST_VIDEO_COMP ## R ## _MASK_16 ", "            \
+    "green_mask = (int) " GST_VIDEO_COMP ## G ## _MASK_16 ", "          \
+    "blue_mask = (int) " GST_VIDEO_COMP ## B ## _MASK_16 ", "           \
+    "width = " GST_VIDEO_SIZE_RANGE ", "                                \
+    "height = " GST_VIDEO_SIZE_RANGE ", "                               \
+    "framerate = " GST_VIDEO_FPS_RANGE
+
+#define __GST_VIDEO_CAPS_MAKE_15(R, G, B)                               \
+    "video/x-raw-rgb, "                                                 \
+    "bpp = (int) 16, "                                                  \
+    "depth = (int) 15, "                                                \
+    "endianness = (int) BYTE_ORDER, "                                   \
+    "red_mask = (int) " GST_VIDEO_COMP ## R ## _MASK_15 ", "            \
+    "green_mask = (int) " GST_VIDEO_COMP ## G ## _MASK_15 ", "          \
+    "blue_mask = (int) " GST_VIDEO_COMP ## B ## _MASK_15 ", "           \
+    "width = " GST_VIDEO_SIZE_RANGE ", "                                \
+    "height = " GST_VIDEO_SIZE_RANGE ", "                               \
+    "framerate = " GST_VIDEO_FPS_RANGE
+
 
 /* 24 bit */
 
@@ -210,30 +276,25 @@ typedef enum {
       
 /* 15/16 bit */
   
-#define GST_VIDEO_CAPS_RGB_16                                           \
-            "video/x-raw-rgb, "                                         \
-            "bpp = (int) 16, "                                          \
-            "depth = (int) 16, "                                        \
-            "endianness = (int) BYTE_ORDER, "                           \
-            "red_mask = (int) " GST_VIDEO_RED_MASK_16 ", "              \
-            "green_mask = (int) " GST_VIDEO_GREEN_MASK_16 ", "          \
-            "blue_mask = (int) " GST_VIDEO_BLUE_MASK_16 ", "            \
-            "width = " GST_VIDEO_SIZE_RANGE ", "                        \
-            "height = " GST_VIDEO_SIZE_RANGE ", "                       \
-            "framerate = " GST_VIDEO_FPS_RANGE
+#define GST_VIDEO_CAPS_RGB_16 \
+    __GST_VIDEO_CAPS_MAKE_16 (1, 2, 3)
 
-#define GST_VIDEO_CAPS_RGB_15                                           \
-            "video/x-raw-rgb, "                                         \
-            "bpp = (int) 16, "                                          \
-            "depth = (int) 15, "                                        \
-            "endianness = (int) BYTE_ORDER, "                           \
-            "red_mask = (int) " GST_VIDEO_RED_MASK_15 ", "              \
-            "green_mask = (int) " GST_VIDEO_GREEN_MASK_15 ", "          \
-            "blue_mask = (int) " GST_VIDEO_BLUE_MASK_15 ", "            \
-            "width = " GST_VIDEO_SIZE_RANGE ", "                        \
-            "height = " GST_VIDEO_SIZE_RANGE ", "                       \
-            "framerate = " GST_VIDEO_FPS_RANGE
+#define GST_VIDEO_CAPS_BGR_16 \
+    __GST_VIDEO_CAPS_MAKE_16 (3, 2, 1)
 
+#define GST_VIDEO_CAPS_RGB_15 \
+    __GST_VIDEO_CAPS_MAKE_15 (1, 2, 3)
+
+#define GST_VIDEO_CAPS_BGR_15 \
+    __GST_VIDEO_CAPS_MAKE_15 (3, 2, 1)
+
+/**
+ * GST_VIDEO_CAPS_YUV:
+ * @fourcc: YUV fourcc format that describes the pixel layout, as string
+ *     (e.g. "I420", "YV12", "YUY2", "AYUV", etc.)
+ *
+ * Generic caps string for YUV video, for use in pad templates.
+ */
 #define GST_VIDEO_CAPS_YUV(fourcc)                                      \
         "video/x-raw-yuv, "                                             \
         "format = (fourcc) " fourcc ", "                                \
@@ -241,6 +302,39 @@ typedef enum {
         "height = " GST_VIDEO_SIZE_RANGE ", "                           \
         "framerate = " GST_VIDEO_FPS_RANGE
 
+/**
+ * GST_VIDEO_CAPS_GRAY8:
+ *
+ * Generic caps string for 8-bit grayscale video, for use in pad templates.
+ *
+ * Since: 0.10.29
+ */
+#define GST_VIDEO_CAPS_GRAY8                                            \
+        "video/x-raw-gray, "                                            \
+        "bpp = (int) 8, "                                               \
+        "depth = (int) 8, "                                             \
+        "width = " GST_VIDEO_SIZE_RANGE ", "                            \
+        "height = " GST_VIDEO_SIZE_RANGE ", "                           \
+        "framerate = " GST_VIDEO_FPS_RANGE
+
+/**
+ * GST_VIDEO_CAPS_GRAY16:
+ * @endianness: endianness as string, ie. either "1234", "4321", "BIG_ENDIAN"
+ *     or "LITTLE_ENDIAN"
+ *
+ * Generic caps string for 16-bit grayscale video, for use in pad templates.
+ *
+ * Since: 0.10.29
+ */
+#define GST_VIDEO_CAPS_GRAY16(endianness)                               \
+        "video/x-raw-gray, "                                            \
+        "bpp = (int) 16, "                                              \
+        "depth = (int) 16, "                                            \
+        "endianness = (int) " endianness ", "                           \
+        "width = " GST_VIDEO_SIZE_RANGE ", "                            \
+        "height = " GST_VIDEO_SIZE_RANGE ", "                           \
+        "framerate = " GST_VIDEO_FPS_RANGE
+
 /* buffer flags */
 
 /**
@@ -291,6 +385,8 @@ gboolean gst_video_parse_caps_framerate (GstCaps *caps,
     int *fps_n, int *fps_d);
 gboolean gst_video_parse_caps_pixel_aspect_ratio (GstCaps *caps,
     int *par_n, int *par_d);
+const char *gst_video_parse_caps_color_matrix (GstCaps * caps);
+const char *gst_video_parse_caps_chroma_site (GstCaps * caps);
 GstCaps * gst_video_format_new_caps (GstVideoFormat format,
     int width, int height, int framerate_n, int framerate_d,
     int par_n, int par_d);
@@ -301,6 +397,7 @@ GstVideoFormat gst_video_format_from_fourcc (guint32 fourcc);
 guint32 gst_video_format_to_fourcc (GstVideoFormat format);
 gboolean gst_video_format_is_rgb (GstVideoFormat format);
 gboolean gst_video_format_is_yuv (GstVideoFormat format);
+gboolean gst_video_format_is_gray (GstVideoFormat format);
 gboolean gst_video_format_has_alpha (GstVideoFormat format);
 int gst_video_format_get_row_stride (GstVideoFormat format, int component,
     int width);
@@ -320,6 +417,14 @@ gboolean gst_video_format_convert (GstVideoFormat format, int width, int height,
 GstEvent *gst_video_event_new_still_frame (gboolean in_still);
 gboolean gst_video_event_parse_still_frame (GstEvent *event, gboolean *in_still);
 
+GstBuffer *gst_video_convert_frame(GstBuffer *buf, const GstCaps *to_caps,
+				   GstClockTime timeout, GError **error);
+
+typedef void (*GstVideoConvertFrameCallback) (GstBuffer *buf, GError *error, gpointer user_data);
+void gst_video_convert_frame_async(GstBuffer *buf, const GstCaps *to_caps,
+				   GstClockTime timeout, GstVideoConvertFrameCallback callback,
+                                   gpointer user_data, GDestroyNotify destroy_notify);
+
 G_END_DECLS
 
 #endif /* __GST_VIDEO_H__ */
