@@ -69,13 +69,6 @@
 #define GST_CAT_DEFAULT audio_rate_debug
 GST_DEBUG_CATEGORY_STATIC (audio_rate_debug);
 
-/* elementfactory information */
-static const GstElementDetails audio_rate_details =
-GST_ELEMENT_DETAILS ("Audio rate adjuster",
-    "Filter/Effect/Audio",
-    "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* GstAudioRate signals and args */
 enum
 {
@@ -133,6 +126,9 @@ static GstElementClass *parent_class = NULL;
 
 /*static guint gst_audio_rate_signals[LAST_SIGNAL] = { 0 }; */
 
+static GParamSpec *pspec_drop = NULL;
+static GParamSpec *pspec_add = NULL;
+
 static GType
 gst_audio_rate_get_type (void)
 {
@@ -163,7 +159,10 @@ gst_audio_rate_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &audio_rate_details);
+  gst_element_class_set_details_simple (element_class,
+      "Audio rate adjuster", "Filter/Effect/Audio",
+      "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
+      "Wim Taymans <wim@fluendo.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_audio_rate_sink_template));
@@ -189,12 +188,12 @@ gst_audio_rate_class_init (GstAudioRateClass * klass)
   g_object_class_install_property (object_class, ARG_OUT,
       g_param_spec_uint64 ("out", "Out", "Number of output samples", 0,
           G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_ADD,
-      g_param_spec_uint64 ("add", "Add", "Number of added samples", 0,
-          G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_DROP,
-      g_param_spec_uint64 ("drop", "Drop", "Number of dropped samples", 0,
-          G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  pspec_add = g_param_spec_uint64 ("add", "Add", "Number of added samples",
+      0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_ADD, pspec_add);
+  pspec_drop = g_param_spec_uint64 ("drop", "Drop", "Number of dropped samples",
+      0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_DROP, pspec_drop);
   g_object_class_install_property (object_class, ARG_SILENT,
       g_param_spec_boolean ("silent", "silent",
           "Don't emit notify for dropped and duplicated frames", DEFAULT_SILENT,
@@ -380,7 +379,9 @@ gst_audio_rate_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
-      /* FIXME, fill last segment */
+      /* Fill segment until the end */
+      if (GST_CLOCK_TIME_IS_VALID (audiorate->src_segment.stop))
+        gst_audio_rate_fill_to_time (audiorate, audiorate->src_segment.stop);
       res = gst_pad_push_event (audiorate->srcpad, event);
       break;
     default:
@@ -495,6 +496,26 @@ gst_audio_rate_convert_segments (GstAudioRate * audiorate)
   return TRUE;
 }
 
+static void
+gst_audio_rate_notify_drop (GstAudioRate * audiorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) audiorate, "drop");
+#else
+  g_object_notify_by_pspec ((GObject *) audiorate, pspec_drop);
+#endif
+}
+
+static void
+gst_audio_rate_notify_add (GstAudioRate * audiorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) audiorate, "add");
+#else
+  g_object_notify_by_pspec ((GObject *) audiorate, pspec_add);
+#endif
+}
+
 static GstFlowReturn
 gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
 {
@@ -556,10 +577,11 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
   GST_LOG_OBJECT (audiorate,
       "in_time:%" GST_TIME_FORMAT ", in_duration:%" GST_TIME_FORMAT
       ", in_size:%u, in_offset:%" G_GUINT64_FORMAT ", in_offset_end:%"
-      G_GUINT64_FORMAT ", ->next_offset:%" G_GUINT64_FORMAT,
-      GST_TIME_ARGS (in_time),
+      G_GUINT64_FORMAT ", ->next_offset:%" G_GUINT64_FORMAT ", ->next_ts:%"
+      GST_TIME_FORMAT, GST_TIME_ARGS (in_time),
       GST_TIME_ARGS (GST_FRAMES_TO_CLOCK_TIME (in_samples, audiorate->rate)),
-      in_size, in_offset, in_offset_end, audiorate->next_offset);
+      in_size, in_offset, in_offset_end, audiorate->next_offset,
+      GST_TIME_ARGS (audiorate->next_ts));
 
   diff = in_time - audiorate->next_ts;
   if (diff <= (GstClockTimeDiff) audiorate->tolerance &&
@@ -569,6 +591,9 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
      * it to next ts and offset and sending */
     GST_LOG_OBJECT (audiorate, "within tolerance %" GST_TIME_FORMAT,
         GST_TIME_ARGS (audiorate->tolerance));
+    /* The outgoing buffer's offset will be set to ->next_offset, we also
+     * need to adjust the offset_end value accordingly */
+    in_offset_end = audiorate->next_offset + in_samples;
     goto send;
   }
 
@@ -626,7 +651,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
       audiorate->add += cursamples;
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "add");
+        gst_audio_rate_notify_add (audiorate);
     }
 
   } else if (in_offset < audiorate->next_offset) {
@@ -641,9 +666,10 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
 
       /* we can drop the buffer completely */
       gst_buffer_unref (buf);
+      buf = NULL;
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "drop");
+        gst_audio_rate_notify_drop (audiorate);
 
       goto beach;
     } else {
@@ -668,7 +694,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
           truncsamples);
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "drop");
+        gst_audio_rate_notify_drop (audiorate);
     }
   }
 
@@ -705,11 +731,15 @@ send:
       GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf));
 
   ret = gst_pad_push (audiorate->srcpad, buf);
+  buf = NULL;
   audiorate->out++;
 
   audiorate->next_offset = in_offset_end;
 beach:
 
+  if (buf)
+    gst_buffer_unref (buf);
+
   gst_object_unref (audiorate);
 
   return ret;
@@ -717,6 +747,8 @@ beach:
   /* ERRORS */
 not_negotiated:
   {
+    gst_buffer_unref (buf);
+
     GST_ELEMENT_ERROR (audiorate, STREAM, FORMAT,
         (NULL), ("pipeline error, format was not negotiated"));
     return GST_FLOW_NOT_NEGOTIATED;
