@@ -37,6 +37,13 @@
 #endif
 #include <gst/interfaces/xoverlay.h>
 
+#if (!GTK_CHECK_VERSION(2, 23, 0) || GTK_CHECK_VERSION(2, 90, 0)) && !GTK_CHECK_VERSION(2, 91, 1)
+#define gtk_combo_box_text_new gtk_combo_box_new_text
+#define gtk_combo_box_text_append_text gtk_combo_box_append_text
+#define gtk_combo_box_text_remove gtk_combo_box_remove_text
+#define GTK_COMBO_BOX_TEXT GTK_COMBO_BOX
+#endif
+
 GST_DEBUG_CATEGORY_STATIC (seek_debug);
 #define GST_CAT_DEFAULT (seek_debug)
 
@@ -78,6 +85,7 @@ gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
 /* the state to go to when stop is pressed */
 #define STOP_STATE      GST_STATE_READY
 
+#define N_GRAD 1000.0
 
 static GList *seekable_pads = NULL;
 static GList *rate_pads = NULL;
@@ -1012,9 +1020,9 @@ format_value (GtkScale * scale, gdouble value)
   gint64 seconds;
   gint64 subseconds;
 
-  real = value * duration / 100;
+  real = value * duration / N_GRAD;
   seconds = (gint64) real / GST_SECOND;
-  subseconds = (gint64) real / (GST_SECOND / 100);
+  subseconds = (gint64) real / (GST_SECOND / N_GRAD);
 
   return g_strdup_printf ("%02" G_GINT64_FORMAT ":%02" G_GINT64_FORMAT ":%02"
       G_GINT64_FORMAT, seconds / 60, seconds % 60, subseconds % 100);
@@ -1195,9 +1203,9 @@ update_fill (gpointer data)
             start, stop);
 
         if (stop != -1)
-          fill = 100.0 * stop / GST_FORMAT_PERCENT_MAX;
+          fill = N_GRAD * stop / GST_FORMAT_PERCENT_MAX;
         else
-          fill = 100.0;
+          fill = N_GRAD;
 
         gtk_range_set_fill_level (GTK_RANGE (hscale), fill);
       }
@@ -1244,7 +1252,7 @@ update_scale (gpointer data)
     duration = position;
 
   if (duration > 0) {
-    set_scale (position * 100.0 / duration);
+    set_scale (position * N_GRAD / duration);
   }
 
   /* FIXME: see make_playerbin2_pipeline() and volume_notify_cb() */
@@ -1314,7 +1322,10 @@ do_seek (GtkWidget * widget)
   GstEvent *s_event;
   GstSeekFlags flags;
 
-  real = gtk_range_get_value (GTK_RANGE (widget)) * duration / 100;
+  real = gtk_range_get_value (GTK_RANGE (widget)) * duration / N_GRAD;
+
+  GST_DEBUG ("value=%f, real=%" G_GINT64_FORMAT,
+      gtk_range_get_value (GTK_RANGE (widget)), real);
 
   flags = 0;
   if (flush_seek)
@@ -1774,11 +1785,11 @@ clear_streams (GstElement * pipeline)
 
   /* remove previous info */
   for (i = 0; i < n_video; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (video_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (video_combo), 0);
   for (i = 0; i < n_audio; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (audio_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (audio_combo), 0);
   for (i = 0; i < n_text; i++)
-    gtk_combo_box_remove_text (GTK_COMBO_BOX (text_combo), 0);
+    gtk_combo_box_text_remove (GTK_COMBO_BOX_TEXT (text_combo), 0);
 
   n_audio = n_video = n_text = 0;
   gtk_widget_set_sensitive (video_combo, FALSE);
@@ -1819,7 +1830,7 @@ update_streams (GstPipeline * pipeline)
       }
       /* find good name for the label */
       name = g_strdup_printf ("video %d", i + 1);
-      gtk_combo_box_append_text (GTK_COMBO_BOX (video_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (video_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (video_checkbox));
@@ -1836,7 +1847,7 @@ update_streams (GstPipeline * pipeline)
       }
       /* find good name for the label */
       name = g_strdup_printf ("audio %d", i + 1);
-      gtk_combo_box_append_text (GTK_COMBO_BOX (audio_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (audio_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (audio_checkbox));
@@ -1865,7 +1876,7 @@ update_streams (GstPipeline * pipeline)
       if (name == NULL)
         name = g_strdup_printf ("text %d", i + 1);
 
-      gtk_combo_box_append_text (GTK_COMBO_BOX (text_combo), name);
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (text_combo), name);
       g_free (name);
     }
     state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (text_checkbox));
@@ -1941,7 +1952,7 @@ init_visualization_features (void)
     name = gst_element_factory_get_longname (entry.factory);
 
     g_array_append_val (vis_entries, entry);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (vis_combo), name);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (vis_combo), name);
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (vis_combo), 0);
 }
@@ -2402,9 +2413,10 @@ static void
 msg_clock_lost (GstBus * bus, GstMessage * message, GstPipeline * data)
 {
   g_print ("clock lost! PAUSE and PLAY to select a new clock\n");
-
-  gst_element_set_state (pipeline, GST_STATE_PAUSED);
-  gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  if (state == GST_STATE_PLAYING) {
+    gst_element_set_state (pipeline, GST_STATE_PAUSED);
+    gst_element_set_state (pipeline, GST_STATE_PLAYING);
+  }
 }
 
 #ifdef HAVE_X
@@ -2437,7 +2449,7 @@ bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * data)
      * shouldn't be done from a non-GUI thread without explicit locking).  */
     g_assert (embed_xid != 0);
 
-    gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (element), embed_xid);
+    gst_x_overlay_set_window_handle (GST_X_OVERLAY (element), embed_xid);
   }
   return GST_BUS_PASS;
 }
@@ -2449,11 +2461,14 @@ handle_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
   if (state < GST_STATE_PAUSED) {
     GtkAllocation allocation;
     GdkWindow *window = gtk_widget_get_window (widget);
-    GtkStyle *style = gtk_widget_get_style (widget);
+    cairo_t *cr;
 
     gtk_widget_get_allocation (widget, &allocation);
-    gdk_draw_rectangle (window, style->black_gc, TRUE, 0, 0,
-        allocation.width, allocation.height);
+    cr = gdk_cairo_create (window);
+    cairo_set_source_rgb (cr, 0, 0, 0);
+    cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+    cairo_fill (cr);
+    cairo_destroy (cr);
   }
   return FALSE;
 }
@@ -2657,8 +2672,9 @@ main (int argc, char **argv)
 
   pipeline_spec = argv[2];
 
-  if (g_strrstr (pipeline_spec, "*") != NULL ||
-      g_strrstr (pipeline_spec, "?") != NULL) {
+  if (g_path_is_absolute (pipeline_spec) &&
+      (g_strrstr (pipeline_spec, "*") != NULL ||
+          g_strrstr (pipeline_spec, "?") != NULL)) {
     paths = handle_wildcards (pipeline_spec);
   } else {
     paths = g_list_prepend (paths, g_strdup (pipeline_spec));
@@ -2732,9 +2748,11 @@ main (int argc, char **argv)
     step = gtk_expander_new ("step options");
     hbox = gtk_hbox_new (FALSE, 0);
 
-    format_combo = gtk_combo_box_new_text ();
-    gtk_combo_box_append_text (GTK_COMBO_BOX (format_combo), "frames");
-    gtk_combo_box_append_text (GTK_COMBO_BOX (format_combo), "time (ms)");
+    format_combo = gtk_combo_box_text_new ();
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format_combo),
+        "frames");
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format_combo),
+        "time (ms)");
     gtk_combo_box_set_active (GTK_COMBO_BOX (format_combo), 0);
     gtk_box_pack_start (GTK_BOX (hbox), format_combo, FALSE, FALSE, 2);
 
@@ -2782,12 +2800,12 @@ main (int argc, char **argv)
 
   /* seek bar */
   adjustment =
-      GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, 100.0, 0.1, 1.0, 1.0));
+      GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, N_GRAD, 0.1, 1.0, 1.0));
   hscale = gtk_hscale_new (adjustment);
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
   gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_RIGHT);
   gtk_range_set_show_fill_level (GTK_RANGE (hscale), TRUE);
-  gtk_range_set_fill_level (GTK_RANGE (hscale), 100.0);
+  gtk_range_set_fill_level (GTK_RANGE (hscale), N_GRAD);
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
   g_signal_connect (hscale, "button_press_event", G_CALLBACK (start_seek),
@@ -2800,9 +2818,9 @@ main (int argc, char **argv)
   if (pipeline_type == 16) {
     /* the playbin2 panel controls for the video/audio/subtitle tracks */
     panel = gtk_hbox_new (FALSE, 0);
-    video_combo = gtk_combo_box_new_text ();
-    audio_combo = gtk_combo_box_new_text ();
-    text_combo = gtk_combo_box_new_text ();
+    video_combo = gtk_combo_box_text_new ();
+    audio_combo = gtk_combo_box_text_new ();
+    text_combo = gtk_combo_box_text_new ();
     gtk_widget_set_sensitive (video_combo, FALSE);
     gtk_widget_set_sensitive (audio_combo, FALSE);
     gtk_widget_set_sensitive (text_combo, FALSE);
@@ -2866,7 +2884,7 @@ main (int argc, char **argv)
         "save a screenshot .png in the current directory");
     g_signal_connect (G_OBJECT (shot_button), "clicked", G_CALLBACK (shot_cb),
         pipeline);
-    vis_combo = gtk_combo_box_new_text ();
+    vis_combo = gtk_combo_box_text_new ();
     g_signal_connect (G_OBJECT (vis_combo), "changed",
         G_CALLBACK (vis_combo_cb), pipeline);
     gtk_widget_set_sensitive (vis_combo, FALSE);
